import 'package:flutter/services.dart';

import 'contact.dart';
import 'message.dart';

enum SmsQueryKind { Inbox, Sent, Draft }

class Thread {
  String _id;
  String _address;
  Contact _contact;
  String _type = "sms";
  List<Message> _messages = [];

  Thread(String id) : this._id = id;

  Thread.fromMessages(List<Message> messages) {
    if (messages == null || messages.length == 0) {
      return;
    }
    this._id = messages[0].threadId;

    for (var msg in messages) {
      if (msg.threadId == _id && msg.address != null) {
        this._address = msg.address;
        break;
      }
    }

    for (var msg in messages) {
      if (msg.threadId == _id) {
        this._messages.add(msg);
      }
    }
  }

  void addMessages(List<Message> messages) {
    this._messages..addAll(messages);
  }

  void addMessage(Message msg) {
    if (msg.threadId == _id) {
      _messages.add(msg);
      if (this._address == null) {
        this._address = msg.address;
      }
    }
  }

  /// Add a message at the start
  void addNewMessage(Message msg) {
    if (msg.threadId == _id) {
      _messages.insert(0, msg);
      if (this._address == null) {
        this._address = msg.address;
      }
    }
  }

  /// Set contact through contact query
  Future findContact() async {
    ContactQuery query = new ContactQuery();
    Contact contact = await query.queryContact(this._address);
    if (contact != null) {
      this._contact = contact;
    }
  }

  /// Get messages from thread
  List<Message> get messages => this._messages;

  /// Get address
  String get address => this._address;

  /// Set messages in thread
  set messages(List<Message> messages) => this._messages = messages;

  /// Get thread id
  String get id => this._id;

  /// Get thread id (for compatibility)
  String get threadId => this._id;

  /// Get contact info
  Contact get contact => this._contact;

  String get type => this._type;

  /// Set contact info
  set contact(Contact contact) => this._contact = contact;
  set address(String address) => this._address = address;
}

class ThreadQuery {
  static ThreadQuery _instance;
  final MethodChannel _channel;
  List<Thread> _moreThreads;

  factory ThreadQuery() {
    if (_instance == null) {
      const platform = const MethodChannel(
          "chatter.romhay.app/querySMS", const JSONMethodCodec());
      _instance = new ThreadQuery._private(platform);
    }
    return _instance;
  }

  ThreadQuery._private(this._channel);

  Future<List<Message>> _querySmsWrapper({int start, int count, String address, int threadId, SmsQueryKind kind: SmsQueryKind.Inbox}) async {
    Map arguments = {};
    if (start != null && start >= 0) {
      arguments["start"] = start;
    }
    if (count != null && count > 0) {
      arguments["count"] = count;
    }
    if (address != null && address.isNotEmpty) {
      arguments["address"] = address;
    }
    if (threadId != null && threadId >= 0) {
      arguments["thread_id"] = threadId;
    }
    String function;
    SmsMessageKind msgKind;
    if (kind == SmsQueryKind.Inbox) {
      function = "getInbox";
      msgKind = SmsMessageKind.Received;
    } else if (kind == SmsQueryKind.Sent) {
      function = "getSent";
      msgKind = SmsMessageKind.Sent;
    } else {
      function = "getDraft";
      msgKind = SmsMessageKind.Draft;
    }
    return await _channel.invokeMethod(function, arguments).then((dynamic val) {
      List<Message> list = [];
      for (Map data in val) {
        Message msg = new Message.fromJson(data);
        msg.kind = msgKind;
        list.add(msg);
      }
      return list;
    });
  }

  Future<List<Message>> querySms({int start, int count, String address, int threadId, List<SmsQueryKind> kinds: const [SmsQueryKind.Inbox], bool sort: true}) async {
    List<Message> result = [];
    for (var kind in kinds) {
      result
        ..addAll(await this._querySmsWrapper(
          start: start,
          count: count,
          address: address,
          threadId: threadId,
          kind: kind,
        ));
    }
    if (sort == true) {
      result.sort((a, b) => a.compareTo(b));
    }
    return (result);
  }

  Future<List<Thread>> queryThreads(List<int> threadsId, {List<SmsQueryKind> kinds: const [SmsQueryKind.Inbox]}) async {
    List<Thread> threads = <Thread>[];
    for (var id in threadsId) {
      final messages = await this.querySms(threadId: id, kinds: kinds);
      final thread = new Thread.fromMessages(messages);
      await thread.findContact();
      threads.add(thread);
    }
    return threads;
  }

  /// Get all SMS
  Future<List<Message>> get getAllSms async {
    return this.querySms(
        kinds: [SmsQueryKind.Sent, SmsQueryKind.Inbox, SmsQueryKind.Draft]);
  }

  set moreThreads(List<Thread> moreThreads) => this._moreThreads = moreThreads;

  /// Get all threads
  Future<List<Thread>> get getAllThreads async {
    List<Message> messages = await this.getAllSms;
    Map<String, List<Message>> filtered = {};
    messages.forEach((msg) {
      if (!filtered.containsKey(msg.threadId)) {
        filtered[msg.threadId] = [];
      }
      filtered[msg.threadId].add(msg);
    });
    List<Thread> threads = <Thread>[];
    for (var k in filtered.keys) {
      final thread = new Thread.fromMessages(filtered[k]);
      await thread.findContact();
      threads.add(thread);
    }
    threads.addAll(this._moreThreads);
    return threads;
  }
}
